--[[
# Element: Raid Role Indicator

Handles the visibility and updating of an indicator based on the unit's raid assignment (main tank or main assist).

## Widget

MainTankAssistIndicator - A `Texture` representing the unit's raid assignment.

## Notes

This element updates by changing the texture.

## Examples

    -- Position and size
    local MainTankAssistIndicator = self:CreateTexture(nil, 'OVERLAY')
    MainTankAssistIndicator:SetSize(16, 16)
    MainTankAssistIndicator:SetPoint('TOPLEFT')

    -- Register it with oUF
    self.MainTankAssistIndicator = MainTankAssistIndicator
--]]

local _, ns = ...
local oUF = ns.oUF
local RUF = RUF or LibStub("AceAddon-3.0"):GetAddon("RUF")

local MAINTANK_ICON = [[Interface\GROUPFRAME\UI-GROUP-MAINTANKICON]]
local MAINASSIST_ICON = [[Interface\GROUPFRAME\UI-GROUP-MAINASSISTICON]]

local function Update(self, event)
	local unit = self.unit

	local element = self.MainTankAssistIndicator

	--[[ Callback: MainTankAssistIndicator:PreUpdate()
	Called before the element has been updated.

	* self - the MainTankAssistIndicator element
	--]]
	if(element.PreUpdate) then
		element:PreUpdate()
	end

	local role, isShown
	if(UnitInRaid(unit) and not UnitHasVehicleUI(unit)) then
		if(GetPartyAssignment('MAINTANK', unit)) then
			isShown = true
			element:SetTexture(MAINTANK_ICON)
			role = 'MAINTANK'
		elseif(GetPartyAssignment('MAINASSIST', unit)) then
			isShown = true
			element:SetTexture(MAINASSIST_ICON)
			role = 'MAINASSIST'
		end
	end

	element:SetShown(isShown)

	if RUF.db.global.TestMode == true then
		element:SetTexture(MAINASSIST_ICON)
		element:Show()
	end

	--[[ Callback: MainTankAssistIndicator:PostUpdate(role)
	Called after the element has been updated.

	* self - the MainTankAssistIndicator element
	* role - the unit's raid assignment (string?)['MAINTANK', 'MAINASSIST']
	--]]
	if(element.PostUpdate) then
		return element:PostUpdate(role)
	end
end

local function Path(self, ...)
	--[[ Override: MainTankAssistIndicator.Override(self, event, ...)
	Used to completely override the internal update function.

	* self  - the parent object
	* event - the event triggering the update (string)
	* ...   - the arguments accompanying the event
	--]]
	return (self.MainTankAssistIndicator.Override or Update)(self, ...)
end

local function ForceUpdate(element)
	return Path(element.__owner, 'ForceUpdate')
end

local function Enable(self)
	local element = self.MainTankAssistIndicator
	if(element) then
		element.__owner = self
		element.ForceUpdate = ForceUpdate

		self:RegisterEvent('GROUP_ROSTER_UPDATE', Path, true)

		return true
	end
end

local function Disable(self)
	local element = self.MainTankAssistIndicator
	if(element) then
		element:Hide()

		self:UnregisterEvent('GROUP_ROSTER_UPDATE', Path)
	end
end

oUF:AddElement('MainTankAssistIndicator', Path, Enable, Disable)
